@@ -4,7 +4,7 @@ import numpy as np
 import scipy.sparse as sp
 import scipy.sparse.linalg as splin
 
-from numpy.testing import assert_allclose
+from numpy.testing import assert_allclose, assert_equal
 
 try:
     import sparse
@@ -15,7 +15,7 @@ pytestmark = pytest.mark.skipif(sparse is None,
                                 reason="pydata/sparse not installed")
 
 
-msg = "pydata/sparse (0.8) does not implement necessary operations"
+msg = "pydata/sparse (0.15.1) does not implement necessary operations"
 
 
 sparse_params = (pytest.param("COO"),
@@ -72,7 +72,13 @@ def test_lsmr(matrices):
     A_dense, A_sparse, b = matrices
     res0 = splin.lsmr(A_dense, b)
     res = splin.lsmr(A_sparse, b)
-    assert_allclose(res[0], res0[0], atol=1.8e-5)
+    assert_allclose(res[0], res0[0], atol=1e-3)
+
+
+# test issue 17012
+def test_lsmr_output_shape():
+    x = splin.lsmr(A=np.ones((10, 1)), b=np.zeros(10), x0=np.ones(1))[0]
+    assert_equal(x.shape, (1,))
 
 
 def test_lsqr(matrices):
@@ -153,7 +159,7 @@ def test_spsolve(matrices):
                        sp.csc_matrix(A_dense))
     x = splin.spsolve(A_sparse, A_sparse)
     assert isinstance(x, type(A_sparse))
-    assert_allclose(x.toarray(), x0.toarray())
+    assert_allclose(x.todense(), x0.todense())
 
 
 def test_splu(matrices):
@@ -166,13 +172,15 @@ def test_splu(matrices):
     assert isinstance(lu.L, sparse_cls)
     assert isinstance(lu.U, sparse_cls)
 
-    Pr = sparse_cls(sp.csc_matrix((np.ones(n), (lu.perm_r, np.arange(n)))))
-    Pc = sparse_cls(sp.csc_matrix((np.ones(n), (np.arange(n), lu.perm_c))))
+    _Pr_scipy = sp.csc_matrix((np.ones(n), (lu.perm_r, np.arange(n))))
+    _Pc_scipy = sp.csc_matrix((np.ones(n), (np.arange(n), lu.perm_c)))
+    Pr = sparse_cls.from_scipy_sparse(_Pr_scipy)
+    Pc = sparse_cls.from_scipy_sparse(_Pc_scipy)
     A2 = Pr.T @ lu.L @ lu.U @ Pc.T
 
-    assert_allclose(A2.toarray(), A_sparse.toarray())
+    assert_allclose(A2.todense(), A_sparse.todense())
 
-    z = lu.solve(A_sparse.toarray())
+    z = lu.solve(A_sparse.todense())
     assert_allclose(z, np.eye(n), atol=1e-10)
 
 
@@ -185,7 +193,7 @@ def test_spilu(matrices):
     assert isinstance(lu.L, sparse_cls)
     assert isinstance(lu.U, sparse_cls)
 
-    z = lu.solve(A_sparse.toarray())
+    z = lu.solve(A_sparse.todense())
     assert_allclose(z, np.eye(len(b)), atol=1e-3)
 
 
@@ -208,14 +216,14 @@ def test_inv(matrices):
     A_dense, A_sparse, b = matrices
     x0 = splin.inv(sp.csc_matrix(A_dense))
     x = splin.inv(A_sparse)
-    assert_allclose(x.toarray(), x0.toarray())
+    assert_allclose(x.todense(), x0.todense())
 
 
 def test_expm(matrices):
     A_dense, A_sparse, b = matrices
     x0 = splin.expm(sp.csc_matrix(A_dense))
     x = splin.expm(A_sparse)
-    assert_allclose(x.toarray(), x0.toarray())
+    assert_allclose(x.todense(), x0.todense())
 
 
 def test_expm_multiply(matrices):
